@@ -2,6 +2,7 @@ import json
 import logging
 import math
 import os
+import re
 import time
 from datetime import timedelta
 
@@ -18,6 +19,29 @@ TOKEN_PRO_TIP = 'ProTip: set GITHUB_API_TOKEN env var as per ' \
                 'https://github.com/dvershinin/lastversion#tips'
 
 
+def set_matching_from_tag(ret, tag):
+    ret = tag
+
+
+def asset_matches(asset, search, regex_matching):
+    """Check if the asset equals to string or satisfies a regular expression
+    Args:
+        asset (dict): asset dict as returned by the API
+        search (str): string or regexp to match asset's name or label with
+        regex_matching (bool): whether search argument is a regexp
+    Returns:
+        bool: Whether match is satisfied
+    """
+    if regex_matching:
+        if asset['label'] and re.search(search, asset['label']):
+            return True
+        if asset['name'] and re.search(search, asset['name']):
+            return True
+    elif search in (asset['label'], asset['name']):
+        return True
+    return False
+
+
 class GitHubRepoSession(ProjectHolder):
     """A class to represent a GitHub project holder."""
 
@@ -129,13 +153,17 @@ class GitHubRepoSession(ProjectHolder):
         else:
             self.api_base = 'https://api.{}'.format(self.DEFAULT_HOSTNAME)
         if '/' not in repo:
-            repo = self.find_repo_by_name_only(repo)
-            if not repo:
-                return
-            self.set_repo(repo)
-            log.info('Using repo {} obtained from search API'.format(self.repo))
-        else:
-            self.set_repo(repo)
+            official_repo = self.try_get_official(repo)
+            if official_repo:
+                repo = official_repo
+                log.info('Using official repo {}'.format(repo))
+            else:
+                repo = self.find_repo_by_name_only(repo)
+                if repo:
+                    log.info('Using repo {} obtained from search API'.format(self.repo))
+                else:
+                    return
+        self.set_repo(repo)
 
     def get_rate_limit_url(self):
         return '{}/rate_limit'.format(self.api_base)
@@ -317,39 +345,32 @@ class GitHubRepoSession(ProjectHolder):
                     log.info('Using commit date')
                     d = node['target']['author']['date']
                 tag_date = parser.parse(d)
-
+                if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                    log.info('The version {} is newer, but is too old!'.format(version))
+                    break
                 if not ret or version > ret['version'] or tag_date > ret['tag_date']:
                     # we always want to return formal release if it exists, cause it has useful
                     # data grab formal release via APi to check for pre-release mark
-                    r = self.repo_query('/releases/tags/{}'.format(tag_name))
-                    if r.status_code == 200:
-                        formal_release = r.json()
-                        if not pre_ok and formal_release['prerelease']:
-                            log.info(
-                                "Found formal release for this tag which is unwanted "
-                                "pre-release: {}.".format(version))
-                            continue
-                        if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                            log.info('The version {} is newer, but is too old!'.format(version))
-                            break
-                        # use full release info
-                        ret = formal_release
-                        ret['tag_name'] = tag_name
-                        ret['tag_date'] = tag_date
-                        ret['version'] = version
-                        ret['type'] = 'graphql'
+                    formal_release = self.get_formal_release_for_tag(tag_name)
+                    if formal_release:
+                        ret = self.set_matching_formal_release(ret, formal_release, version, pre_ok)
                     else:
-                        if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                            log.info('The version {} is newer, but is too old!'.format(version))
-                            break
-                        ret = {
-                            'tag_name': tag_name,
-                            'tag_date': tag_date,
-                            'version': version,
-                            'type': 'graphql'
-                        }
+                        if not self.having_asset:
+                            ret = {
+                                'tag_name': tag_name,
+                                'tag_date': tag_date,
+                                'version': version,
+                                'type': 'graphql'
+                            }
         return ret
 
+    def get_formal_release_for_tag(self, tag):
+        r = self.repo_query('/releases/tags/{}'.format(tag))
+        if r.status_code == 200:
+            # noinspection SpellCheckingInspection
+            return r.json()
+        return None
+
     # finding in tags requires paging through ALL of them, because the API does not list them
     # in order of recency, thus this is very slow
     # in: current release to be returned, output: newer release to be returned
@@ -375,25 +396,9 @@ class GitHubRepoSession(ProjectHolder):
                 # rare case: if upstream filed formal pre-release that passes as stable
                 # version (tag is 1.2.3 instead of 1.2.3b) double check if pre-release
                 # TODO handle API failure here as it may result in "false positive"?
-                release_for_tag = None
-                if not pre_ok:
-                    r = self.repo_query('/releases/tags/{}'.format(tag_name))
-                    if r.status_code == 200:
-                        # noinspection SpellCheckingInspection
-                        if r.json()['prerelease']:
-                            log.info(
-                                "Found formal release for this tag which is unwanted "
-                                "pre-release: {}.".format(version))
-                            continue
-                        release_for_tag = r.json()
-
-                log.info("Setting version as current selection: {}.".format(version))
+                release_for_tag = self.get_formal_release_for_tag(tag_name)
                 if release_for_tag:
-                    ret = release_for_tag
-                    ret['tag_name'] = tag_name
-                    ret['tag_date'] = parser.parse(release_for_tag['published_at'])
-                    ret['version'] = version
-                    ret['type'] = 'release'
+                    ret = self.set_matching_formal_release(ret, release_for_tag, version, pre_ok)
                 else:
                     ret = t
                     ret['tag_name'] = tag_name
@@ -485,43 +490,30 @@ class GitHubRepoSession(ProjectHolder):
                     'Tag {} does not contain newer version than we already found'.format(tag_name)
                 )
                 continue
+            tag_date = parser.parse(tag['updated'])
+            if ret and tag_date + timedelta(days=365) < ret['tag_date']:
+                log.info('The version {} is newer, but is too old!'.format(version))
+                break
             # we always want to return formal release if it exists, cause it has useful data
             # grab formal release via APi to check for pre-release mark
-            r = self.repo_query('/releases/tags/{}'.format(tag_name))
-            if r.status_code == 200:
-                log.info('Got formal release for tag {}'.format(tag_name))
-                formal_release = r.json()
-                if not pre_ok and formal_release['prerelease']:
-                    log.info(
-                        "Found formal release for this tag which is unwanted "
-                        "pre-release: {}.".format(version))
-                    continue
-                tag_date = parser.parse(tag['updated'])
-                if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                    log.info('The version {} is newer, but is too old!'.format(version))
-                    break
+            formal_release = self.get_formal_release_for_tag(tag_name)
+            if formal_release:
                 # use full release info
-                ret = formal_release
-                ret['tag_name'] = tag_name
-                ret['tag_date'] = tag_date
-                ret['version'] = version
-                ret['type'] = 'feed'
+                ret = self.set_matching_formal_release(ret, formal_release, version, pre_ok)
             else:
+                if self.having_asset:
+                    continue
                 log.info('No formal release for tag {}'.format(tag_name))
-                tag_date = parser.parse(tag['updated'])
-                if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                    log.info('The version {} is newer, but is too old!'.format(version))
-                    break
-                ret = tag
-                ret['tag_name'] = tag_name
-                ret['tag_date'] = tag_date
-                ret['version'] = version
-                ret['type'] = 'feed'
+                tag['tag_name'] = tag_name
+                tag['tag_date'] = tag_date
+                tag['version'] = version
+                tag['type'] = 'feed'
                 # remove keys which are non-jsonable
                 # TODO use those (pop returns them)
-                ret.pop('updated_parsed', None)
-                ret.pop('published_parsed', None)
-            log.info("Selected version as current selection: {}.".format(version))
+                tag.pop('updated_parsed', None)
+                tag.pop('published_parsed', None)
+                ret = tag
+                log.info("Selected version as current selection: {}.".format(version))
 
         # we are good with release from feeds only without looking at the API
         # simply because feeds list stuff in order of recency
@@ -541,15 +533,11 @@ class GitHubRepoSession(ProjectHolder):
             # ideally we disable ETag validation for this endpoint completely
             r = self.repo_query('/releases/latest')
             if r.status_code == 200:
-                tag_name = r.json()['tag_name']
-                version = self.sanitize_version(tag_name, pre_ok, major)
+                formal_release = r.json()
+                version = self.sanitize_version(formal_release['tag_name'], pre_ok, major)
                 if version:
-                    log.info("Selected version as current selection: {}.".format(version))
-                    ret = r.json()
-                    ret['tag_name'] = tag_name
-                    ret['tag_date'] = parser.parse(r.json()['published_at'])
-                    ret['version'] = version
-                    ret['type'] = 'releases-latest'
+                    ret = self.set_matching_formal_release(ret, formal_release, version,
+                                                           pre_ok, 'releases-latest')
         else:
             r = self.repo_query('/releases')
             if r.status_code == 200:
@@ -559,12 +547,13 @@ class GitHubRepoSession(ProjectHolder):
                     if not version:
                         continue
                     if not ret or version > ret['version']:
-                        log.info("Selected version as current selection: {}.".format(version))
-                        ret = release
-                        ret['tag_name'] = tag_name
-                        ret['tag_date'] = parser.parse(release['published_at'])
-                        ret['version'] = version
-                        ret['type'] = 'release'
+                        ret = self.set_matching_formal_release(ret, release, version, pre_ok)
+
+        if self.having_asset:
+            # only formal releases which we enumerated above already, have assets
+            # so there is no point looking in the tags/graphql below
+            # return whatever we got
+            return ret
 
         # formal release may not exist at all, or be "late/old" in case
         # actual release is only a simple tag so let's try /tags
@@ -575,3 +564,51 @@ class GitHubRepoSession(ProjectHolder):
             ret = self.find_in_tags(ret, pre_ok, major)
 
         return ret
+
+    def set_matching_formal_release(self, ret, formal_release, version, pre_ok,
+                                    data_type='release'):
+        """Set current release selection to this formal release if matching conditions."""
+        if not pre_ok and formal_release['prerelease']:
+            log.info(
+                "Found formal release for this tag which is unwanted "
+                "pre-release: {}.".format(version))
+            return ret
+        if self.having_asset:
+            if 'assets' not in formal_release or not formal_release['assets']:
+                log.info('Skipping this release due to no assets.')
+                return ret
+            if self.having_asset is not True:
+                regex_matching = False
+                search = self.having_asset
+                if search.startswith('~'):
+                    search = r'{}'.format(search.lstrip('~'))
+                    regex_matching = True
+                found_asset = False
+                for asset in formal_release['assets']:
+                    if asset_matches(asset, search, regex_matching=regex_matching):
+                        found_asset = True
+                        break
+                if not found_asset:
+                    log.info('Desired asset not found in the release.')
+                    return ret
+        # formal_release['tag_name'] = tag_name
+        formal_release['tag_date'] = parser.parse(formal_release['published_at'])
+        formal_release['version'] = version
+        formal_release['type'] = data_type
+        log.info("Selected version as current selection: {}.".format(formal_release['version']))
+        return formal_release
+
+    def try_get_official(self, repo):
+        """Check existence of repo/repo
+
+        Returns:
+            str: updated repo
+        """
+        official_repo = "{repo}/{repo}".format(repo=repo)
+        log.info('Checking existence of {}'.format(official_repo))
+        url = '{}/repos/{}'.format(self.api_base, official_repo)
+        r = self.get(url)
+        if r.status_code == 200:
+            return official_repo
+        return None
+
