@@ -37,7 +37,8 @@ __self__ = "dvershinin/lastversion"
 
 
 def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
-           short_urls=False, major=None, only=None, at=None):
+           short_urls=False, major=None, only=None, at=None,
+           having_asset=None):
     """Find latest release version for a project.
 
     Args:
@@ -51,6 +52,8 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
         pre_ok (bool): Specifies whether pre-releases can be accepted as newer version.
         at (str): Specifies repo hosting more precisely, only useful if repo argument was
                   specified as one word.
+        having_asset (Union[str, bool]): Only consider releases with the given asset.
+                                         Pass `True` for any asset
 
     Returns:
         Version: Newer version object, if found and `output_format` is `version`.
@@ -88,8 +91,11 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
             upstream_github = None
             upstream_name = None
             current_version = None
+            spec_repo = None
             for l in f.readlines():
-                if l.startswith('%global upstream_github'):
+                if l.startswith('%global lastversion_repo'):
+                    spec_repo = l.split(' ')[2].strip()
+                elif l.startswith('%global upstream_github'):
                     upstream_github = l.split(' ')[2].strip()
                 elif l.startswith('%global upstream_name'):
                     upstream_name = l.split(' ')[2].strip()
@@ -101,9 +107,10 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
                     repo_data['module_of'] = True
                 elif l.startswith('Version:') and not current_version:
                     current_version = l.split('Version:')[1].strip()
-            if not upstream_github:
-                log.critical('%upstream_github macro not found. Please prepare your spec file '
-                             'using instructions: https://github.com/dvershinin/lastversion/wiki/Preparing-RPM-spec-files')
+            if not upstream_github and not spec_repo:
+                log.critical('Neither %upstream_github nor %lastversion_repo macros were found. '
+                             'Please prepare your spec file using instructions: '
+                             'https://lastversion.getpagespeed.com/spec-preparing.html')
             if not current_version:
                 log.critical('Did not find neither Version: nor %upstream_version in the spec file')
                 sys.exit(1)
@@ -120,8 +127,12 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
             else:
                 repo_data['name'] = name
                 repo_data['spec_name'] = '%{name}'
-            repo = "{}/{}".format(upstream_github, repo_data['name'])
-            log.info('Discovered GitHub repo {} from .spec file'.format(repo))
+            if upstream_github:
+                repo = "{}/{}".format(upstream_github, repo_data['name'])
+                log.info('Discovered GitHub repo {} from .spec file'.format(repo))
+            else:
+                repo = spec_repo
+                log.info('Discovered explicit repo {} from .spec file'.format(repo))
 
     if (not at or '/' in repo) and at != 'helm_chart':
         # find the right hosting for this repo
@@ -130,6 +141,7 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
         project_holder = HolderFactory.HOLDERS[at](repo, hostname=None)
 
     project_holder.set_only(only)
+    project_holder.set_having_asset(having_asset)
 
     # we are completely "offline" for 1 hour, not even making conditional requests
     # heuristic=ExpiresAfter(hours=1)   <- make configurable
@@ -276,6 +288,7 @@ def get_rpm_packager():
 
 
 def update_spec(repo, res, sem='minor'):
+    print(res['version'])
     if 'current_version' not in res or res['current_version'] < res['version']:
         log.info('Updating spec {} with semantic {}'.format(repo, sem))
         if 'current_version' in res and len(res['version'].release) >= 3:
@@ -313,9 +326,9 @@ def update_spec(repo, res, sem='minor'):
                 m = re.match(version_tag_regex, ln)
                 out.append('Version:' + m.group(1) + str(res['version']))
             elif ln.startswith('%changelog') and packager:
-                import datetime
-                today = datetime.date.today()
-                today = today.strftime('%a %b %d %Y')
+                from datetime import datetime
+                now = datetime.utcnow()
+                today = now.strftime('%a %b %d %Y')
                 out.append(ln.rstrip())
                 out.append('* {} {}'.format(today, packager))
                 out.append('- upstream release v{}'.format(res['version']))
@@ -378,6 +391,9 @@ def main():
                                                        "projects inside")
     parser.add_argument('--filter', metavar='REGEX', help="Filters --assets result by a regular "
                                                           "expression")
+    parser.add_argument('--having-asset', metavar='ASSET',
+                        help="Only consider releases with this asset",
+                        nargs='?', const=True)
     parser.add_argument('-su', '--shorter-urls', dest='shorter_urls', action='store_true',
                         help='A tiny bit shorter URLs produced')
     parser.add_argument('--at', dest='at',
@@ -389,7 +405,8 @@ def main():
     parser.add_argument('--version', action=VersionAction)
     parser.set_defaults(validate=True, verbose=False, format='version',
                         pre=False, assets=False, newer_than=False, filter=False,
-                        shorter_urls=False, major=None, assumeyes=False, at=None)
+                        shorter_urls=False, major=None, assumeyes=False, at=None,
+                        having_asset=None)
     args = parser.parse_args()
 
     if args.repo == "self":
@@ -468,15 +485,17 @@ def main():
         if args.format != 'assets':
             args.format = 'source'
 
-    base_compare = parse_version(args.repo)
-    if args.newer_than and base_compare:
-        print(max([args.newer_than, base_compare]))
-        sys.exit(2 if base_compare <= args.newer_than else 0)
+    if args.newer_than:
+        base_compare = parse_version(args.repo)
+        if base_compare:
+            print(max([args.newer_than, base_compare]))
+            sys.exit(2 if base_compare <= args.newer_than else 0)
 
     # other action are either getting release or doing something with release (extend get action)
     try:
         res = latest(args.repo, args.format, args.pre, args.filter,
-                     args.shorter_urls, args.major, args.only, args.at)
+                     args.shorter_urls, args.major, args.only, args.at,
+                     having_asset=args.having_asset)
     except (ApiCredentialsError, BadProjectError) as error:
         sys.stderr.write(str(error) + os.linesep)
         if isinstance(error, ApiCredentialsError) and "GITHUB_API_TOKEN" not in os.environ and \
